<?php


class Apps extends Tools_Plugins_Abstract {

    /**
     * List of action that should be allowed to specific roles
     *
     * By default all of actions of your plugin are available to the guest user
     * @var array
     */
    const SERVICE_CONSTANT_CONTACT = 'constantContact';

    const SERVICE_TYPE_EMAIL = 'email';

    const SERVICE_TWILIO = 'twilio';

    const SERVICE_TYPE_SMS = 'sms';

    const SECTION_APPS = 'socialposter';

    const SMS_FROM_TYPE_FORM = 'form';

    const SMS_FROM_TYPE_NEW_ORDER = 'newOrder';

    const SMS_FROM_TYPE_TRACKING_NUMBER = 'trackingNumber';

    /**
     * Status when sms sent to admin
     */
    const SMS_OWNER_TYPE_ADMIN = 'admin';

    /**
     * Status when sms sent to the end user
     */
    const SMS_OWNER_TYPE_USER = 'user';

    /**
     * secure token
     */
    const APPS_SECURE_TOKEN = 'AppsToken';

    private $_helpHashMap  = array(
        self::SECTION_APPS => 'cms-applications.html'
    );

    protected $_securedActions = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'secured'
        )
    );

    /**
     * layout
     * @var null
     */
    protected $_layout = null;

    /**
     * Init method.
     *
     * Use this method to init your plugin's data and variables
     * Use this method to init specific helpers, view, etc...
     */
    private $_websiteConfig = '';

    private $_categoryAlias = array('email' => 'Email marketing', 'sms' => 'SMS and phone services', 'crm' => 'CRM');

    protected function _init() {
        $this->_websiteConfig = Zend_Controller_Action_HelperBroker::getStaticHelper('config')->getConfig();

        $this->_layout = new Zend_Layout();
        $this->_layout->setLayoutPath(Zend_Layout::getMvcInstance()->getLayoutPath());

        if(($scriptPaths = Zend_Layout::getMvcInstance()->getView()->getScriptPaths()) !== false) {
            $this->_view->setScriptPath($scriptPaths);
        }
        $this->_view->addScriptPath(__DIR__ . '/system/views/');
    }

    public function appsConfigAction(){
        if(Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $appsSettingsMapper = Apps_Models_Mapper_AppsSettingsMapper::getInstance();
            if($this->_request->isPost()){
                $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
                $tokenValid = Tools_System_Tools::validateToken($secureToken, self::APPS_SECURE_TOKEN);
                if (!$tokenValid) {
                    $this->_responseHelper->fail('');
                }
                $data = $this->_request->getParams();
                $appsSettingsModel = new Apps_Models_Models_AppsSettingsModel();
                if(isset($data['service'])){
                    $status = ($data['status'] === 'enable') ? 0 : 1;
                    $appsSettingsModel->setServiceName($data['service']);
                    $appsSettingsModel->setStatus($status);
                    $appsSettingsModel->setCategory($data['category']);
                    $appsSettingsMapper->save($appsSettingsModel);
                    $this->_responseHelper->success($this->_translator->translate('Changed'));
                }
                $this->_responseHelper->fail();
            }else{
                $this->_view->localServices = $appsSettingsMapper->selectConfig();
                $categoryAndServices = Apps::apiCall('GET', 'apps', array('services'));
                if(isset($categoryAndServices['done']) && $categoryAndServices['done'] === false) {
                    $categoryAndServices = null;
                }
                if($categoryAndServices !== null) {
                    if(isset($categoryAndServices['services']['categoryInfo'])){
                        $this->_view->categoryInfo = $categoryAndServices['services']['categoryInfo'];
                        unset($categoryAndServices['services']['categoryInfo']);
                    }
                    $this->_view->categoryAndServices = $categoryAndServices;
                    $this->_view->categoryAlias = $this->_categoryAlias;
                    $allForms = Application_Model_Mappers_FormMapper::getInstance()->fetchAll();
                    $this->_view->forms = $allForms;
                    $this->_view->emailFormsView = $this->_view->render('emailForm.phtml');
                }

            }
            $this->_view->helpSection = self::SECTION_APPS;
            $this->_view->hashMap     = $this->_helpHashMap;
            $this->_layout->content = $this->_view->render('config.phtml');
            echo $this->_layout->render();

        }
    }

    public function chooseFormListAction(){
        if(Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS) && $this->_request->isPost()) {
            $formName = $this->_request->getParam('checkedForm');
            $service  = $this->_request->getParam('serviceName');
            $existingForms = Apps_Models_Mapper_AppsSystemFormMapper::getInstance()->getByFormNameService($formName, $service);
            $localLists = array();
            if(!empty($existingForms)){
                $localLists = explode(',', $existingForms[0]->getLists());
            }
            if(isset($service)){
                $contactLists = Apps::apiCall('GET', 'apps', array($service.'List'));
                if(isset($contactLists[$service.'List']) && is_array($contactLists[$service.'List'])){
                    foreach($contactLists[$service.'List'] as $list){
                        $lists[$list['id']] = $list['name'];
                    }
                    $this->_view->lists = $lists;
                }else{
                    $this->_responseHelper->fail();
                }
            }
            $this->_responseHelper->success(array('localLists'=> $localLists, 'list' => $lists));
        }
    }

    public function saveSystemFormAction(){
        if(Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS) && $this->_request->isPost()) {
            $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::APPS_SECURE_TOKEN);
            if (!$tokenValid) {
                $this->_responseHelper->fail('');
            }
            $data = $this->_request->getParams();
            $appsSystemFormsMapper = Apps_Models_Mapper_AppsSystemFormMapper::getInstance();
            if(isset($data['lists']) && !empty($data['lists'])){
                $appsSystemFormsModel  = new Apps_Models_Models_AppsSystemFormModel();
                $appsSystemFormsModel->setForm($data['formName']);
                $appsSystemFormsModel->setLists(implode(',',array_values($data['lists'])));
                $appsSystemFormsModel->setService($data['service']);
                $appsSystemFormsMapper->save($appsSystemFormsModel);
            }elseif(isset($data['deleteList']) && isset($data['service']) && $data['formName'] && $data['deleteList'] == 1 ){
                $appsSystemFormsMapper->deleteList($data['service'], $data['formName']);
            }else{
                $this->_responseHelper->fail();
            }
            $this->_responseHelper->success('');
        }
    }

    public function getEnabledServicesDashboardAction(){
        if(Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $data = $this->_request->getParams();
            $enabledServices = Apps_Models_Mapper_AppsSettingsMapper::getInstance()->selectConfigByStatusCategory('1', self::SERVICE_TYPE_EMAIL);
            if(!empty($enabledServices)){
                $this->_responseHelper->success(array('clients' => $data['customers'], 'enabledServices' => $enabledServices));
            }
            $this->_responseHelper->fail();
        }
    }

    public function getServiceAction(){
        if(Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $data = $this->_request->getParams();
            if(isset($data['serviceName'])){
                $services = Apps::apiCall('GET', 'apps', array($data['serviceName'].'List'));
                if(isset($services[$data['serviceName'].'List'])){
                    foreach($services[$data['serviceName'].'List'] as $list){
                        $lists[$list['id']] = $list['name'];
                    }
                    $this->_responseHelper->success(array('list' =>$lists));
                }
                $this->_responseHelper->fail();
            }
            $this->_responseHelper->fail();
        }
    }

    public function sendServicesDashboardAction(){
        if(Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS) && $this->_request->isPost()) {
            $serviceData = $this->_request->getParams();
            if(isset($serviceData['customers']) && isset($serviceData['service']) && isset($serviceData['lists'])){
                $data['services'] = array($serviceData['service'] => array('type' => self::SERVICE_TYPE_EMAIL));
                $userDbtable = new Application_Model_DbTable_User();
                $where  = $userDbtable->getAdapter()->quoteInto('id IN (?)', explode(',', $serviceData['customers']));
                $select = $userDbtable->getAdapter()->select()->from('user', array('email', 'full_name'))->where($where);
                $users  = $userDbtable->getAdapter()->fetchPairs($select);
                if(!empty($users)){
                    $data['services'][$serviceData['service']]['clients'] = $users;
                    $data['services'][$serviceData['service']]['lists'] = explode(',', $serviceData['lists']);
                    $response = Apps::apiCall('POST', 'apps', array($serviceData['service'].'Client'), $data);
                    if($response){
                        $this->_responseHelper->success('');
                    }
                    $this->_responseHelper->fail();

                }
            }
        }
    }

    public static function apiCall($methodType, $methodName, $command, $data = null) {
        $websiteConfig = Zend_Controller_Action_HelperBroker::getStaticHelper('config')->getConfig();
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $websiteUrl = $websiteHelper->getUrl();
        if(isset($websiteConfig['sambaToken']) && (isset($websiteConfig['websiteId'])) ) {
            $url = parse_url($websiteUrl);
            $data['websiteUrl'] = $url['host'];
            $data['websiteId'] = $websiteConfig['websiteId'];
            $data['sambaToken'] = $websiteConfig['sambaToken'];
            $data['command'] = $command;
            $seosambaRequest = Tools_Factory_PluginFactory::createPlugin('api',array(), array('websiteUrl' => $websiteUrl));
            return $seosambaRequest::request($methodType, $methodName, $data);
        }
    }

}
